@@ -1,19 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 2005, Nickolay V. Shmyrev <nshmyrev@yandex.ru>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -161,11 +148,16 @@ djvu_document_load (PpsDocument *document,
 	gint n_files;
 	gint i;
 	gchar *base;
+	gboolean success = TRUE;
+
+	g_rw_lock_writer_lock (&djvu_document->rwlock);
 
 	/* FIXME: We could actually load uris  */
 	filename = g_filename_from_uri (uri, NULL, error);
-	if (!filename)
+	if (!filename) {
+		g_rw_lock_writer_unlock (&djvu_document->rwlock);
 		return FALSE;
+	}
 
 #ifdef __APPLE__
 	doc = ddjvu_document_create_by_filename_utf8 (djvu_document->d_context, filename, TRUE);
@@ -179,6 +171,7 @@ djvu_document_load (PpsDocument *document,
 		                     PPS_DOCUMENT_ERROR,
 		                     PPS_DOCUMENT_ERROR_INVALID,
 		                     _ ("DjVu document has incorrect format"));
+		g_rw_lock_writer_unlock (&djvu_document->rwlock);
 		return FALSE;
 	}
 
@@ -198,6 +191,7 @@ djvu_document_load (PpsDocument *document,
 		ddjvu_document_release (djvu_document->d_document);
 		djvu_document->d_document = NULL;
 
+		g_rw_lock_writer_unlock (&djvu_document->rwlock);
 		return FALSE;
 	}
 
@@ -214,6 +208,7 @@ djvu_document_load (PpsDocument *document,
 		ddjvu_document_release (djvu_document->d_document);
 		djvu_document->d_document = NULL;
 
+		g_rw_lock_writer_unlock (&djvu_document->rwlock);
 		return FALSE;
 	}
 
@@ -268,10 +263,11 @@ djvu_document_load (PpsDocument *document,
 		                     _ ("The document is composed of several files. "
 		                        "One or more of these files cannot be accessed."));
 
-		return FALSE;
+		success = FALSE;
 	}
 
-	return TRUE;
+	g_rw_lock_writer_unlock (&djvu_document->rwlock);
+	return success;
 }
 
 static gboolean
@@ -280,18 +276,30 @@ djvu_document_save (PpsDocument *document,
                     GError **error)
 {
 	DjvuDocument *djvu_document = DJVU_DOCUMENT (document);
+	gboolean result;
 
-	return pps_xfer_uri_simple (djvu_document->uri, uri, error);
+	g_rw_lock_writer_lock (&djvu_document->rwlock);
+	result = pps_xfer_uri_simple (djvu_document->uri, uri, error);
+	g_rw_lock_writer_unlock (&djvu_document->rwlock);
+
+	return result;
 }
 
 int
 djvu_document_get_n_pages (PpsDocument *document)
 {
 	DjvuDocument *djvu_document = DJVU_DOCUMENT (document);
+	int n_pages;
+
+	g_rw_lock_reader_lock (&djvu_document->rwlock);
 
 	g_return_val_if_fail (djvu_document->d_document, 0);
 
-	return ddjvu_document_get_pagenum (djvu_document->d_document);
+	n_pages = ddjvu_document_get_pagenum (djvu_document->d_document);
+
+	g_rw_lock_reader_unlock (&djvu_document->rwlock);
+
+	return n_pages;
 }
 
 static void
@@ -326,10 +334,14 @@ djvu_document_get_page_size (PpsDocument *document,
 {
 	DjvuDocument *djvu_document = DJVU_DOCUMENT (document);
 
+	g_rw_lock_reader_lock (&djvu_document->rwlock);
+
 	g_return_if_fail (djvu_document->d_document);
 
 	document_get_page_size (djvu_document, page->index,
 	                        width, height, NULL);
+
+	g_rw_lock_reader_unlock (&djvu_document->rwlock);
 }
 
 static cairo_surface_t *
@@ -348,6 +360,8 @@ djvu_document_render (PpsDocument *document,
 	double page_width, page_height;
 	gint transformed_width, transformed_height;
 
+	g_rw_lock_reader_lock (&djvu_document->rwlock);
+
 	d_page = ddjvu_page_create_by_pageno (djvu_document->d_document, rc->page->index);
 
 	while (!ddjvu_page_decoding_done (d_page))
@@ -412,6 +426,8 @@ djvu_document_render (PpsDocument *document,
 		cairo_surface_mark_dirty (surface);
 	}
 
+	g_rw_lock_reader_unlock (&djvu_document->rwlock);
+
 	return surface;
 }
 
@@ -423,16 +439,22 @@ djvu_document_get_page_label (PpsDocument *document,
 	const gchar *title = NULL;
 	gchar *label = NULL;
 
+	g_rw_lock_reader_lock (&djvu_document->rwlock);
+
 	g_assert (page->index >= 0 && page->index < djvu_document->n_pages);
 
-	if (djvu_document->fileinfo_pages == NULL)
+	if (djvu_document->fileinfo_pages == NULL) {
+		g_rw_lock_reader_unlock (&djvu_document->rwlock);
 		return NULL;
+	}
 
 	title = djvu_document->fileinfo_pages[page->index].title;
 
 	if (!g_str_has_suffix (title, ".djvu"))
 		label = g_strdup (title);
 
+	g_rw_lock_reader_unlock (&djvu_document->rwlock);
+
 	return label;
 }
 
@@ -442,10 +464,12 @@ djvu_document_get_thumbnail (PpsDocument *document,
 {
 	DjvuDocument *djvu_document = DJVU_DOCUMENT (document);
 	GdkPixbuf *pixbuf, *rotated_pixbuf;
-	gdouble page_width = 0., page_height = 0.;
+	gdouble page_width, page_height;
 	gint thumb_width, thumb_height;
 	guchar *pixels;
 
+	g_rw_lock_reader_lock (&djvu_document->rwlock);
+
 	g_return_val_if_fail (djvu_document->d_document, NULL);
 
 	djvu_document_get_page_size (PPS_DOCUMENT (djvu_document), rc->page,
@@ -471,6 +495,8 @@ djvu_document_get_thumbnail (PpsDocument *document,
 	rotated_pixbuf = gdk_pixbuf_rotate_simple (pixbuf, 360 - rc->rotation);
 	g_object_unref (pixbuf);
 
+	g_rw_lock_reader_unlock (&djvu_document->rwlock);
+
 	return rotated_pixbuf;
 }
 
@@ -480,11 +506,13 @@ djvu_document_get_thumbnail_surface (PpsDocument *document,
 {
 	DjvuDocument *djvu_document = DJVU_DOCUMENT (document);
 	cairo_surface_t *surface, *rotated_surface;
-	gdouble page_width = 0., page_height = 0.;
+	gdouble page_width, page_height;
 	gint thumb_width, thumb_height;
 	gchar *pixels;
 	gint thumbnail_rendered;
 
+	g_rw_lock_reader_lock (&djvu_document->rwlock);
+
 	g_return_val_if_fail (djvu_document->d_document, NULL);
 
 	djvu_document_get_page_size (PPS_DOCUMENT (djvu_document), rc->page,
@@ -506,6 +534,7 @@ djvu_document_get_thumbnail_surface (PpsDocument *document,
 	                                             djvu_document->d_format,
 	                                             cairo_image_surface_get_stride (surface),
 	                                             pixels);
+	g_rw_lock_reader_unlock (&djvu_document->rwlock);
 
 	if (!thumbnail_rendered) {
 		cairo_surface_destroy (surface);
@@ -531,11 +560,14 @@ djvu_document_get_info (PpsDocument *document)
 	miniexp_t anno;
 	PpsDocumentInfo *info;
 
+	g_rw_lock_reader_lock (&djvu_document->rwlock);
+
 	info = pps_document_info_new ();
 
 	anno = ddjvu_document_get_anno (djvu_document->d_document, 1);
 	if (anno == miniexp_nil) {
 		ddjvu_miniexp_release (djvu_document->d_document, anno);
+		g_rw_lock_reader_unlock (&djvu_document->rwlock);
 		return info;
 	}
 
@@ -545,6 +577,9 @@ djvu_document_get_info (PpsDocument *document)
 	}
 
 	ddjvu_miniexp_release (djvu_document->d_document, anno);
+
+	g_rw_lock_reader_unlock (&djvu_document->rwlock);
+
 	return info;
 }
 
@@ -572,6 +607,7 @@ djvu_document_finalize (GObject *object)
 	ddjvu_format_release (djvu_document->d_format);
 	ddjvu_format_release (djvu_document->thumbs_format);
 	g_free (djvu_document->uri);
+	g_rw_lock_clear (&djvu_document->rwlock);
 
 	G_OBJECT_CLASS (djvu_document_parent_class)->finalize (object);
 }
@@ -705,12 +741,19 @@ djvu_selection_get_selection_region (PpsSelection *selection,
 	DjvuDocument *djvu_document = DJVU_DOCUMENT (selection);
 	gdouble page_width, page_height;
 	gdouble scale_x, scale_y;
+	cairo_region_t *region;
+
+	g_rw_lock_reader_lock (&djvu_document->rwlock);
 
 	document_get_page_size (djvu_document, rc->page->index, &page_width, &page_height, NULL);
 	pps_render_context_compute_scales (rc, page_width, page_height, &scale_x, &scale_y);
 
-	return djvu_get_selection_region (djvu_document, rc->page->index,
-	                                  scale_x, scale_y, points);
+	region = djvu_get_selection_region (djvu_document, rc->page->index,
+	                                    scale_x, scale_y, points);
+
+	g_rw_lock_reader_unlock (&djvu_document->rwlock);
+
+	return region;
 }
 
 static gchar *
@@ -724,10 +767,14 @@ djvu_selection_get_selected_text (PpsSelection *selection,
 	PpsRectangle rectangle;
 	gchar *text;
 
+	g_rw_lock_reader_lock (&djvu_document->rwlock);
+
 	document_get_page_size (djvu_document, page->index, NULL, &height, &dpi);
 	djvu_convert_to_doc_rect (&rectangle, points, height, dpi);
 	text = djvu_text_copy (djvu_document, page->index, &rectangle);
 
+	g_rw_lock_reader_unlock (&djvu_document->rwlock);
+
 	if (text == NULL)
 		text = g_strdup ("");
 
@@ -747,6 +794,9 @@ djvu_document_text_get_text_mapping (PpsDocumentText *document_text,
 {
 	DjvuDocument *djvu_document = DJVU_DOCUMENT (document_text);
 	PpsRectangle points;
+	cairo_region_t *region;
+
+	g_rw_lock_reader_lock (&djvu_document->rwlock);
 
 	points.x1 = 0;
 	points.y1 = 0;
@@ -754,8 +804,12 @@ djvu_document_text_get_text_mapping (PpsDocumentText *document_text,
 	document_get_page_size (djvu_document, page->index,
 	                        &points.x2, &points.y2, NULL);
 
-	return djvu_get_selection_region (djvu_document, page->index,
-	                                  1.0, 1.0, &points);
+	region = djvu_get_selection_region (djvu_document, page->index,
+	                                    1.0, 1.0, &points);
+
+	g_rw_lock_reader_unlock (&djvu_document->rwlock);
+
+	return region;
 }
 
 static gchar *
@@ -766,6 +820,8 @@ djvu_document_text_get_text (PpsDocumentText *selection,
 	miniexp_t page_text;
 	gchar *text = NULL;
 
+	g_rw_lock_reader_lock (&djvu_document->rwlock);
+
 	while ((page_text = ddjvu_document_get_pagetext (djvu_document->d_document,
 	                                                 page->index,
 	                                                 "char")) == miniexp_dummy)
@@ -780,6 +836,9 @@ djvu_document_text_get_text (PpsDocumentText *selection,
 		djvu_text_page_free (tpage);
 		ddjvu_miniexp_release (djvu_document->d_document, page_text);
 	}
+
+	g_rw_lock_reader_unlock (&djvu_document->rwlock);
+
 	return text;
 }
 
@@ -822,9 +881,12 @@ djvu_document_file_exporter_end (PpsFileExporter *exporter)
 
 	DjvuDocument *djvu_document = DJVU_DOCUMENT (exporter);
 
+	g_rw_lock_reader_lock (&djvu_document->rwlock);
+
 	FILE *fn = fopen (djvu_document->ps_filename, "w");
 	if (fn == NULL) {
 		g_warning ("Cannot open file “%s”.", djvu_document->ps_filename);
+		g_rw_lock_reader_unlock (&djvu_document->rwlock);
 		return;
 	}
 
@@ -836,6 +898,7 @@ djvu_document_file_exporter_end (PpsFileExporter *exporter)
 	}
 
 	fclose (fn);
+	g_rw_lock_reader_unlock (&djvu_document->rwlock);
 }
 
 static PpsFileExporterCapabilities
@@ -873,6 +936,7 @@ djvu_document_init (DjvuDocument *djvu_document)
 	djvu_document->opts = g_string_new ("");
 
 	djvu_document->d_document = NULL;
+	g_rw_lock_init (&djvu_document->rwlock);
 }
 
 static GList *
@@ -890,6 +954,8 @@ djvu_document_find_find_text (PpsDocumentFind *document,
 
 	g_return_val_if_fail (text != NULL, NULL);
 
+	g_rw_lock_reader_lock (&djvu_document->rwlock);
+
 	while ((page_text = ddjvu_document_get_pagetext (djvu_document->d_document,
 	                                                 page->index,
 	                                                 "char")) == miniexp_dummy)
@@ -912,6 +978,9 @@ djvu_document_find_find_text (PpsDocumentFind *document,
 		djvu_text_page_free (tpage);
 		ddjvu_miniexp_release (djvu_document->d_document, page_text);
 	}
+
+	g_rw_lock_reader_unlock (&djvu_document->rwlock);
+
 	if (!matches)
 		return NULL;
 
