@@ -1,21 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  *  Copyright (C) 2000-2003 Marco Pesenti Gritti
  *  Copyright © 2021 Christian Persch
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
  */
 
 #pragma once
@@ -206,3 +192,6 @@ PPS_PUBLIC
 const gchar *pps_document_license_get_web_statement (PpsDocumentLicense *license);
 
 G_END_DECLS
+
+/* Automatic Cleanup */
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (PpsDocumentInfo, pps_document_info_free)
