@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* pps-signature.c
  * this file is part of papers, a gnome document viewer
  *
  * Copyright (C) 2024 Jan-Michael Brummer <jan-michael.brummer1@volkswagen.de>
- *
- * Papers is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * Papers is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include "config.h"
@@ -197,6 +184,8 @@ pps_signature_class_init (PpsSignatureClass *klass)
 	                                                      G_PARAM_READWRITE |
 	                                                          G_PARAM_CONSTRUCT_ONLY |
 	                                                          G_PARAM_STATIC_STRINGS));
+
+	g_object_class->finalize = pps_signature_finalize;
 }
 
 void
@@ -240,7 +229,7 @@ pps_signature_set_rect (PpsSignature *self,
 {
 	PpsSignaturePrivate *priv = GET_SIG_PRIVATE (self);
 
-	g_clear_pointer (&priv->rect, g_free);
+	g_clear_object (&priv->rect);
 	priv->rect = pps_rectangle_copy ((PpsRectangle *) rectangle);
 }
 
@@ -481,7 +470,7 @@ PpsSignature *
 pps_signature_new (PpsSignatureStatus status, PpsCertificateInfo *info)
 {
 	return g_object_new (PPS_TYPE_SIGNATURE,
-	                     "status", PPS_SIGNATURE_STATUS_INVALID,
+	                     "status", status,
 	                     "certificate-info", info,
 	                     NULL);
 }
