@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* this file is part of papers, a gnome document viewer
  *
  *  Copyright (C) 2008 Carlos Garcia Campos <carlosgc@gnome.org>
  *  Copyright (C) 2005 Red Hat, Inc
- *
- * Papers is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * Papers is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #pragma once
@@ -116,9 +103,8 @@ typedef enum {
 	PPS_PAGE_DATA_INCLUDE_TEXT_LOG_ATTRS = 1 << 5,
 	PPS_PAGE_DATA_INCLUDE_IMAGES = 1 << 6,
 	PPS_PAGE_DATA_INCLUDE_FORMS = 1 << 7,
-	PPS_PAGE_DATA_INCLUDE_ANNOTS = 1 << 8,
-	PPS_PAGE_DATA_INCLUDE_MEDIA = 1 << 9,
-	PPS_PAGE_DATA_INCLUDE_ALL = (1 << 10) - 1
+	PPS_PAGE_DATA_INCLUDE_MEDIA = 1 << 8,
+	PPS_PAGE_DATA_INCLUDE_ALL = (1 << 9) - 1
 } PpsJobPageDataFlags;
 
 struct _PpsJobPageData {
@@ -130,7 +116,6 @@ struct _PpsJobPageData {
 	PpsMappingList *link_mapping;
 	PpsMappingList *image_mapping;
 	PpsMappingList *form_field_mapping;
-	PpsMappingList *annot_mapping;
 	PpsMappingList *media_mapping;
 	cairo_region_t *text_mapping;
 	gchar *text;
@@ -309,9 +294,6 @@ void pps_job_load_set_password_save (PpsJobLoad *job,
 PPS_PUBLIC
 GPasswordSave pps_job_load_get_password_save (PpsJobLoad *job);
 PPS_PUBLIC
-void pps_job_load_set_load_flags (PpsJobLoad *job,
-                                  PpsDocumentLoadFlags flags);
-PPS_PUBLIC
 PpsDocument *pps_job_load_get_loaded_document (PpsJobLoad *job);
 
 /* PpsJobSave */
