@@ -1,5 +1,4 @@
 use crate::deps::*;
-use crate::window::WindowRunMode;
 
 use std::cell::Cell;
 use std::collections::VecDeque;
@@ -10,9 +9,8 @@ use glib::{UserDirectory, Variant, VariantTy};
 use gtk::{PrintOperationResult, TextDirection};
 
 use papers_document::{
-    Annotation, AnnotationMarkup, AnnotationText, AnnotationTextMarkup, AnnotationsSaveMask,
-    Attachment, DocumentAnnotations, DocumentForms, Link, LinkAction, LinkActionType, LinkDest,
-    Outlines,
+    Annotation, AnnotationMarkup, AnnotationText, AnnotationTextMarkup, Attachment,
+    DocumentAnnotations, DocumentForms, Link, LinkAction, LinkActionType, LinkDest, Outlines,
 };
 use papers_view::{History, PageLayout, SizingMode};
 
@@ -20,6 +18,7 @@ use crate::application;
 
 /// Action handling of this widget
 mod actions;
+mod enums;
 mod io;
 mod print;
 /// Signal handling callbacks of this widget
@@ -148,6 +147,10 @@ mod imp {
         // Search
         pub(super) search_context: RefCell<Option<papers_view::SearchContext>>,
 
+        // Undo
+        #[template_child]
+        pub(super) undo_context: TemplateChild<papers_view::UndoContext>,
+
         // Annotations
         #[template_child]
         pub(super) annots_context: TemplateChild<papers_view::AnnotationsContext>,
@@ -204,9 +207,6 @@ mod imp {
                 obj.as_ref()
                     .imp()
                     .set_action_enabled("find-previous", false);
-                obj.as_ref()
-                    .imp()
-                    .set_action_enabled("add-highlight-annotation", false);
                 obj.as_ref().imp().set_action_enabled("copy", false);
             }
         }
@@ -237,6 +237,8 @@ mod imp {
                 .set_allow_links_change_zoom(allow_links_change_zoom);
 
             self.view.set_model(&self.model);
+            self.view
+                .set_annotations_context(&self.annots_context.get());
 
             self.default_settings.delay();
             self.setup_default();
@@ -268,8 +270,6 @@ mod imp {
 
             self.find_sidebar.set_search_context(Some(&search_context));
             self.view.set_search_context(&search_context);
-            self.view
-                .set_annotations_context(&self.annots_context.get());
             self.search_context.replace(Some(search_context));
         }
     }
@@ -562,9 +562,9 @@ mod imp {
 
             self.sidebar_stack
                 .set_visible_child(&self.find_sidebar.get());
+            self.find_sidebar.grab_focus();
             self.document_action_group
                 .change_action_state("show-sidebar", &true.into());
-            self.find_sidebar.grab_focus();
             self.set_action_enabled("find-next", true);
             self.set_action_enabled("find-previous", true);
         }
@@ -776,7 +776,8 @@ mod imp {
 
 glib::wrapper! {
     pub struct PpsDocumentView(ObjectSubclass<imp::PpsDocumentView>)
-        @extends gtk::Widget, adw::BreakpointBin;
+        @extends gtk::Widget, adw::BreakpointBin,
+        @implements gtk::Accessible, gtk::Buildable, gtk::ConstraintTarget;
 }
 
 impl PpsDocumentView {
@@ -834,9 +835,8 @@ impl PpsDocumentView {
         document: &Document,
         metadata: Option<&papers_view::Metadata>,
         dest: Option<&LinkDest>,
-        mode: WindowRunMode,
     ) {
-        self.imp().open_document(document, metadata, dest, mode);
+        self.imp().open_document(document, metadata, dest);
     }
 
     pub fn reload_document(&self, document: Option<&Document>) {
@@ -846,10 +846,6 @@ impl PpsDocumentView {
     pub fn error_message(&self, error: Option<&glib::Error>, msg: &str) {
         self.imp().error_message(error, msg);
     }
-
-    pub fn focus_view(&self) {
-        self.imp().view.grab_focus();
-    }
 }
 
 impl Default for PpsDocumentView {
