@@ -1,5 +1,6 @@
 use super::*;
 
+use crate::document_view::enums::AnnotationColor;
 use gtk::gdk::gdk_pixbuf;
 use papers_document::{AnnotationTextMarkupType, DocumentImages, DocumentSignatures};
 use papers_view::annotations_context::AddAnnotationData;
@@ -65,6 +66,7 @@ impl imp::PpsDocumentView {
                 .add_child(&self.annot_menu_child.get(), "palette");
         } else {
             self.set_action_enabled("add-text-annotation", false);
+            self.set_action_enabled("annot-textmarkup-type", false);
         }
 
         let can_sign = document
@@ -296,22 +298,24 @@ impl imp::PpsDocumentView {
                 .build(),
             gio::ActionEntryBuilder::new("annot-color")
                 .parameter_type(Some(glib::VariantTy::STRING))
-                .state(glib::Variant::from("yellow"))
+                .state(glib::Variant::from(&AnnotationColor::Yellow.to_string()))
                 .change_state(glib::clone!(
-                    #[weak(rename_to = _obj)]
+                    #[weak(rename_to = obj)]
                     self,
                     move |_, action, state| {
+                        obj.cmd_annot_color(state.unwrap().str().unwrap());
                         action.set_state(state.unwrap());
                     }
                 ))
                 .build(),
-            gio::ActionEntryBuilder::new("annot-style")
+            gio::ActionEntryBuilder::new("annot-textmarkup-type")
                 .parameter_type(Some(glib::VariantTy::STRING))
                 .state(glib::Variant::from("highlight"))
                 .change_state(glib::clone!(
-                    #[weak(rename_to = _obj)]
+                    #[weak(rename_to = obj)]
                     self,
                     move |_, action, state| {
+                        obj.cmd_annot_textmarkup_type(state.unwrap().str().unwrap());
                         action.set_state(state.unwrap());
                     }
                 ))
@@ -399,6 +403,24 @@ impl imp::PpsDocumentView {
                     move |_, _, _| obj.view.copy()
                 ))
                 .build(),
+            gio::ActionEntryBuilder::new("undo")
+                .activate(glib::clone!(
+                    #[weak(rename_to = obj)]
+                    self,
+                    move |_, _, _| {
+                        obj.undo_context.undo();
+                    }
+                ))
+                .build(),
+            gio::ActionEntryBuilder::new("redo")
+                .activate(glib::clone!(
+                    #[weak(rename_to = obj)]
+                    self,
+                    move |_, _, _| {
+                        obj.undo_context.redo();
+                    }
+                ))
+                .build(),
             gio::ActionEntryBuilder::new("caret-navigation")
                 .activate(glib::clone!(
                     #[weak(rename_to = obj)]
@@ -493,15 +515,6 @@ impl imp::PpsDocumentView {
                     }
                 ))
                 .build(),
-            gio::ActionEntryBuilder::new("add-highlight-annotation")
-                .activate(glib::clone!(
-                    #[weak(rename_to = obj)]
-                    self,
-                    move |_, _, _| {
-                        obj.cmd_add_highlight_annotation();
-                    }
-                ))
-                .build(),
             gio::ActionEntryBuilder::new("add-text-annotation")
                 .activate(glib::clone!(
                     #[weak(rename_to = obj)]
@@ -590,25 +603,15 @@ impl imp::PpsDocumentView {
                     self,
                     move |_, _, _| {
                         let selected_text = obj.view.selected_text().filter(|t| !t.is_empty());
-
+                        obj.document_action_group
+                            .change_action_state("toggle-find", &true.into());
                         if let Some(selected_text) = selected_text {
                             obj.search_context
                                 .borrow()
                                 .as_ref()
                                 .unwrap()
                                 .set_search_term(&selected_text);
-                            obj.document_action_group
-                                .change_action_state("toggle-find", &true.into());
                             obj.find_restart();
-                        } else {
-                            let active = obj
-                                .sidebar_stack
-                                .visible_child()
-                                .is_some_and(|w| w == *obj.find_sidebar)
-                                && obj.find_sidebar.focus_child().is_some();
-
-                            obj.document_action_group
-                                .change_action_state("toggle-find", &(!active).into());
                         }
                     }
                 ))
@@ -697,7 +700,7 @@ impl imp::PpsDocumentView {
         let properties = PpsPropertiesWindow::new();
 
         properties.set_document(self.model.document());
-        properties.present(Some(self.obj().as_ref()));
+        AdwDialogExt::present(&properties, Some(self.obj().as_ref()));
     }
 
     fn cmd_escape(&self) {
@@ -718,6 +721,83 @@ impl imp::PpsDocumentView {
         }
     }
 
+    fn cmd_annot_color(&self, color: &str) {
+        let rgba = AnnotationColor::from(color).to_rgba();
+
+        self.set_annot_textmarkup_icon_color(&rgba);
+
+        let annot = self.annot.borrow().clone();
+        if let Some(annot) = annot {
+            if annot.rgba() != rgba {
+                annot.set_rgba(&rgba);
+            }
+        }
+    }
+
+    fn cmd_annot_textmarkup_type(&self, markup_type: &str) {
+        let markup_type = match markup_type {
+            "highlight" => Some(AnnotationTextMarkupType::Highlight),
+            "squiggly" => Some(AnnotationTextMarkupType::Squiggly),
+            "strikethrough" => Some(AnnotationTextMarkupType::StrikeOut),
+            "underline" => Some(AnnotationTextMarkupType::Underline),
+            "none" => None,
+            _ => panic!("unknown markup_type {markup_type}"),
+        };
+
+        if markup_type.is_none() {
+            return;
+        }
+
+        let annot = self.annot.borrow().clone();
+        if let Some(annot) = annot {
+            if let Some(annot) = annot.dynamic_cast_ref::<AnnotationTextMarkup>() {
+                let markup_type = markup_type.expect("no markup_type, but an annot is set");
+                if annot.markup_type() != markup_type {
+                    annot.set_markup_type(markup_type);
+                }
+            }
+        } else if self.view.has_selection() {
+            let selections = self.view.selections();
+            for sel in selections.iter() {
+                let mut start_point = papers_document::Point::new();
+                let mut end_point = papers_document::Point::new();
+                let markup_type = markup_type.expect("no markup_type, but an annot is set");
+                start_point.set_x(sel.rect().x1());
+                start_point.set_y(sel.rect().y1());
+                end_point.set_x(sel.rect().x2());
+                end_point.set_y(sel.rect().y2());
+                let annot = self.annots_context.add_annotation_sync(
+                    sel.page(),
+                    papers_document::AnnotationType::TextMarkup,
+                    &start_point,
+                    &end_point,
+                    &self.rgba_from_annot_color(),
+                    AddAnnotationData::TextMarkup(markup_type),
+                );
+                self.annot.replace(annot.as_ref().cloned());
+            }
+        }
+    }
+
+    fn set_annot_textmarkup_icon_color(&self, color: &gdk::RGBA) {
+        let css_color = color.to_string();
+        let css = format!("#annot-type-container {{--annot-textmarkup-color: {css_color}; }}");
+
+        let provider = gtk::CssProvider::new();
+        provider.load_from_string(&css);
+
+        match gdk::Display::default() {
+            Some(display) => {
+                gtk::style_context_add_provider_for_display(
+                    &display,
+                    &provider,
+                    gtk::STYLE_PROVIDER_PRIORITY_APPLICATION,
+                );
+            }
+            _ => glib::g_critical!("", "Could not find a display"),
+        }
+    }
+
     fn rotate(&self, degree: i32) {
         let rotation = self.model.rotation() + degree;
         self.model.set_rotation(rotation);
@@ -804,10 +884,6 @@ impl imp::PpsDocumentView {
 
                 let target_file = Self::create_file_from_uri_for_format(&uri, &format);
 
-                let document = obj.document().unwrap();
-
-                document.doc_mutex_lock();
-
                 let pixbuf = obj
                     .document()
                     .and_dynamic_cast::<DocumentImages>()
@@ -815,8 +891,6 @@ impl imp::PpsDocumentView {
                     .and_then(|d| d.image(&image))
                     .unwrap();
 
-                document.doc_mutex_unlock();
-
                 if let Err(e) = obj.save_file(&target_file, |target| {
                     pixbuf.savev(target.path().unwrap(), &format.name().unwrap(), &[])
                 }) {
@@ -832,17 +906,13 @@ impl imp::PpsDocumentView {
             return;
         };
         let clipboard = self.obj().clipboard();
-        let document = self.document().unwrap();
 
-        document.doc_mutex_lock();
-
-        let pixbuf = document
-            .dynamic_cast_ref::<papers_document::DocumentImages>()
+        let pixbuf = self
+            .document()
+            .and_dynamic_cast::<papers_document::DocumentImages>()
             .unwrap()
             .image(&image);
 
-        document.doc_mutex_unlock();
-
         clipboard.set_texture(&gdk::Texture::for_pixbuf(&pixbuf.unwrap()))
     }
 
@@ -852,47 +922,22 @@ impl imp::PpsDocumentView {
         let rgba = dialog.rgba();
         let opacity = dialog.opacity();
         let popup_is_open = dialog.popup_open();
-        let mut mask = AnnotationsSaveMask::NONE;
         let _notify = annot.freeze_notify();
 
         // Set annotations changes
         if let Some(annot) = annot.dynamic_cast_ref::<AnnotationMarkup>() {
-            if annot.set_label(&author) {
-                mask |= AnnotationsSaveMask::LABEL;
-            }
-            if annot.set_rgba(&rgba) {
-                mask |= AnnotationsSaveMask::COLOR;
-            }
-            if annot.set_opacity(opacity) {
-                mask |= AnnotationsSaveMask::OPACITY;
-            }
-            if annot.set_popup_is_open(popup_is_open) {
-                mask |= AnnotationsSaveMask::POPUP_IS_OPEN;
-            }
+            annot.set_label(&author);
+            annot.set_rgba(&rgba);
+            annot.set_opacity(opacity);
+            annot.set_popup_is_open(popup_is_open);
         }
 
         if let Some(annot) = annot.dynamic_cast_ref::<AnnotationText>() {
-            let icon = dialog.text_icon();
-            if annot.set_icon(icon) {
-                mask |= AnnotationsSaveMask::TEXT_ICON;
-            }
+            annot.set_icon(dialog.text_icon());
         }
 
         if let Some(annot) = annot.dynamic_cast_ref::<AnnotationTextMarkup>() {
-            let markup_type = dialog.markup_type();
-            if annot.set_markup_type(markup_type) {
-                mask |= AnnotationsSaveMask::TEXT_MARKUP_TYPE;
-            }
-        }
-
-        if !mask.is_empty() {
-            if let Some(document) = self.document() {
-                if let Some(doc_annot) = document.dynamic_cast_ref::<DocumentAnnotations>() {
-                    document.doc_mutex_lock();
-                    doc_annot.save_annotation(&annot, mask);
-                    document.doc_mutex_unlock();
-                }
-            }
+            annot.set_markup_type(dialog.markup_type());
         }
     }
 
@@ -903,15 +948,15 @@ impl imp::PpsDocumentView {
 
         let dialog = PpsAnnotationPropertiesDialog::new(&annot);
 
-        dialog.connect_response(
-            Some("apply"),
-            glib::clone!(
+        dialog.connect_closure(
+            "changed",
+            true,
+            glib::closure_local!(
                 #[weak(rename_to = obj)]
                 self,
-                move |dialog, _| obj.annot_properties_dialog_response(dialog)
+                move |dialog| { obj.annot_properties_dialog_response(dialog) }
             ),
         );
-
         dialog.present(Some(self.obj().as_ref()));
     }
 
@@ -928,10 +973,11 @@ impl imp::PpsDocumentView {
 
         let toast = adw::Toast::builder().title(&msg).timeout(5).build();
 
+        self.toast_overlay.dismiss_all();
         self.toast_overlay.add_toast(toast);
     }
 
-    fn annot_color_rgba(&self) -> gdk::RGBA {
+    fn rgba_from_annot_color(&self) -> gdk::RGBA {
         let binding = self
             .document_action_group
             .lookup_action("annot-color")
@@ -939,55 +985,7 @@ impl imp::PpsDocumentView {
             .state()
             .unwrap();
         let color = binding.str().unwrap();
-        match color {
-            "yellow" => gdk::RGBA::parse("#f5c211").unwrap(),
-            "orange" => gdk::RGBA::parse("#ff7800").unwrap(),
-            "red" => gdk::RGBA::parse("#ed333b").unwrap(),
-            "purple" => gdk::RGBA::parse("#c061cb").unwrap(),
-            "blue" => gdk::RGBA::parse("#3584e4").unwrap(),
-            "green" => gdk::RGBA::parse("#33d17a").unwrap(),
-            _ => panic!("unknown color {color}"),
-        }
-    }
-
-    fn annot_style(&self) -> AnnotationTextMarkupType {
-        use papers_document::AnnotationTextMarkupType::*;
-
-        let binding = self
-            .document_action_group
-            .lookup_action("annot-style")
-            .unwrap()
-            .state()
-            .unwrap();
-        let style = binding.str().unwrap();
-
-        match style {
-            "highlight" => Highlight,
-            "squiggly" => Squiggly,
-            "strikethrough" => StrikeOut,
-            "underline" => Underline,
-            _ => panic!("unknown style {style}"),
-        }
-    }
-
-    fn cmd_add_highlight_annotation(&self) {
-        let selections = self.view.selections();
-        for sel in selections.iter() {
-            let mut start_point = papers_document::Point::new();
-            let mut end_point = papers_document::Point::new();
-            start_point.set_x(sel.rect().x1());
-            start_point.set_y(sel.rect().y1());
-            end_point.set_x(sel.rect().x2());
-            end_point.set_y(sel.rect().y2());
-            _ = self.annots_context.add_annotation_sync(
-                sel.page(),
-                papers_document::AnnotationType::TextMarkup,
-                &start_point,
-                &end_point,
-                &self.annot_color_rgba(),
-                AddAnnotationData::TextMarkup(self.annot_style()),
-            );
-        }
+        AnnotationColor::from(color).to_rgba()
     }
 
     fn cmd_add_text_annotation(&self) {
@@ -1005,13 +1003,13 @@ impl imp::PpsDocumentView {
             y = rect.y();
         };
 
-        if let Some(mark) = self.view.mark_for_view_point(x.into(), y.into()) {
+        if let Some(doc_point) = self.view.document_point_for_view_point(x.into(), y.into()) {
             _ = self.annots_context.add_annotation_sync(
-                mark.page_index(),
+                doc_point.page_index(),
                 papers_document::AnnotationType::Text,
-                &mark.doc_point(),
-                &mark.doc_point(),
-                &self.annot_color_rgba(),
+                &doc_point.point_on_page(),
+                &doc_point.point_on_page(),
+                &self.rgba_from_annot_color(),
                 AddAnnotationData::None,
             );
         };
