@@ -1,6 +1,7 @@
 use super::*;
 
-use papers_document::DocumentPermissions;
+use crate::document_view::enums::AnnotationColor;
+use papers_document::{AnnotationTextMarkupType, DocumentPermissions};
 
 #[gtk::template_callbacks]
 impl imp::PpsDocumentView {
@@ -188,8 +189,8 @@ impl imp::PpsDocumentView {
     }
 
     #[template_callback]
-    fn sidebar_annots_annot_activated(&self, annot_mapping: &papers_document::Mapping) {
-        self.view.focus_annotation(annot_mapping)
+    fn sidebar_annots_annot_activated(&self, annot: &papers_document::Annotation) {
+        self.view.focus_annotation(annot)
     }
 
     // view
@@ -280,7 +281,7 @@ impl imp::PpsDocumentView {
         // The goto-remote action should not reference the file itself. If it
         // does, simply ignore it. Ideally, we would not launch a new instance
         // but open a new tab or a new window, but that's not possible until
-        // https://gitlab.gnome.org/GNOME/Incubator/papers/-/issues/104 is fixed
+        // https://gitlab.gnome.org/GNOME/papers/-/issues/104 is fixed
         if target != path {
             application::spawn(
                 Some(&gio::File::for_path(target)),
@@ -324,7 +325,7 @@ impl imp::PpsDocumentView {
                 self.document_action_group.activate_action("save-as", None)
             }
             _ => {
-                glib::g_warning!("", "Unimplemented named action: {}, please post a bug report in Papers Gitlab (https://gitlab.gnome.org/GNOME/Incubator/papers/issues) with a testcase.", name);
+                glib::g_warning!("", "Unimplemented named action: {}, please post a bug report in Document Viewer Gitlab (https://gitlab.gnome.org/GNOME/papers/issues) with a testcase.", name);
             }
         }
     }
@@ -477,10 +478,14 @@ impl imp::PpsDocumentView {
 
     pub(super) fn view_menu_annot_popup(&self, annot: Option<&papers_document::Annotation>) {
         let mut show_annot_props = false;
+        let mut is_annot_textmarkup = false;
         let mut show_attachment = false;
+        let mut markup_type = "none";
+        let has_selection = self.view.has_selection();
 
         if let Some(annot) = annot {
             show_annot_props = annot.is::<papers_document::AnnotationMarkup>();
+            is_annot_textmarkup = annot.is::<papers_document::AnnotationTextMarkup>();
 
             let attachment = annot
                 .dynamic_cast_ref::<papers_document::AnnotationAttachment>()
@@ -498,12 +503,37 @@ impl imp::PpsDocumentView {
             .unwrap_or_default()
             && annot.is_some();
 
-        self.annot.replace(annot.cloned());
-
         self.set_action_enabled("annot-properties", show_annot_props);
+        self.set_action_enabled(
+            "annot-textmarkup-type",
+            is_annot_textmarkup || has_selection,
+        );
         self.set_action_enabled("remove-annot", can_remove_annots);
         self.set_action_enabled("open-attachment", show_attachment);
         self.set_action_enabled("save-attachment", show_attachment);
+
+        if is_annot_textmarkup {
+            markup_type = match annot
+                .unwrap()
+                .dynamic_cast_ref::<AnnotationTextMarkup>()
+                .unwrap()
+                .markup_type()
+            {
+                AnnotationTextMarkupType::Highlight => "highlight",
+                AnnotationTextMarkupType::Squiggly => "squiggly",
+                AnnotationTextMarkupType::StrikeOut => "strikethrough",
+                AnnotationTextMarkupType::Underline => "underline",
+                _ => panic!("unknown markup type"),
+            };
+        }
+
+        if let Some(annot) = annot.cloned() {
+            let color = AnnotationColor::from(annot.rgba()).to_string();
+            self.set_action_state("annot-color", &glib::Variant::from(color.as_str()));
+        };
+        self.set_action_state("annot-textmarkup-type", &glib::Variant::from(markup_type));
+
+        self.annot.replace(annot.cloned());
     }
 
     #[template_callback]
@@ -580,7 +610,7 @@ impl imp::PpsDocumentView {
             .unwrap_or_default();
 
         self.set_action_enabled("copy", has_selection);
-        self.set_action_enabled("add-highlight-annotation", has_selection && can_annotate);
+        self.set_action_enabled("annot-textmarkup-type", can_annotate);
     }
 
     #[template_callback]
@@ -661,12 +691,8 @@ impl imp::PpsDocumentView {
             _ => panic!(),
         };
 
-        let show_zoom_fit_best = self.model.sizing_mode() != SizingMode::Automatic;
-        if self.zoom_fit_best_revealer.focus_child().is_some() && !show_zoom_fit_best {
-            self.view.grab_focus();
-        }
         self.zoom_fit_best_revealer
-            .set_reveal_child(show_zoom_fit_best);
+            .set_reveal_child(self.model.sizing_mode() != SizingMode::Automatic);
 
         action.change_state(&state.into());
 
@@ -743,13 +769,13 @@ impl imp::PpsDocumentView {
         properties.set_visible_page_name(Some("signatures"));
         properties.set_document(self.document());
 
-        properties.present(Some(self.obj().as_ref()));
+        AdwDialogExt::present(&properties, Some(self.obj().as_ref()));
     }
 
     #[template_callback]
     fn signature_rect_too_small(&self, response: &str) {
         if response == "sign" {
-            self.certificate_save_as_dialog();
+            self.create_certificate_selection();
             return;
         }
 
@@ -811,7 +837,6 @@ impl imp::PpsDocumentView {
 
         self.signature_page.set(page);
         self.signature_bounding_box.replace(Some(*rect));
-        signature.set_rect(rect);
 
         // Calculate font size for main (right) signature text
         let Some(font_size) = Self::calculate_font_size(
@@ -837,6 +862,8 @@ impl imp::PpsDocumentView {
 
         signature.set_left_font_size(font_size as i32);
 
+        signature.set_rect(rect);
+
         self.certificate_save_as_dialog();
     }
 }
