@@ -591,7 +591,7 @@ mod imp {
 
                                     obj.list_view.scroll_to(
                                         pos,
-                                        gtk::ListScrollFlags::SELECT,
+                                        gtk::ListScrollFlags::FOCUS | gtk::ListScrollFlags::SELECT,
                                         None,
                                     );
 
@@ -837,6 +837,7 @@ mod imp {
                 .ellipsize(gtk::pango::EllipsizeMode::End)
                 .hexpand(true)
                 .wrap(true)
+                .wrap_mode(gtk::pango::WrapMode::WordChar)
                 .lines(3)
                 .margin_top(6)
                 .margin_bottom(6)
@@ -845,7 +846,6 @@ mod imp {
 
             let index = gtk::Label::builder()
                 .xalign(1.0)
-                .ellipsize(gtk::pango::EllipsizeMode::Middle)
                 .max_width_chars(7)
                 .css_classes(["dim-label"])
                 .build();
@@ -953,8 +953,8 @@ mod imp {
 
             let markup = item.markup();
 
-            outlines.set_tooltip_markup(markup.as_ref().map(|m| m.as_str().trim_end()));
-            outlines.set_markup(markup.unwrap_or_default().trim_end());
+            outlines.set_tooltip_markup(markup.as_ref().map(|m| m.as_str()));
+            outlines.set_markup(&markup.unwrap_or_default());
             page_label.set_label(&item.label().unwrap_or_default());
         }
 
@@ -982,7 +982,8 @@ mod imp {
 
 glib::wrapper! {
     pub struct PpsSidebarLinks(ObjectSubclass<imp::PpsSidebarLinks>)
-        @extends PpsSidebarPage, adw::Bin, gtk::Widget;
+        @extends PpsSidebarPage, adw::Bin, gtk::Widget,
+        @implements gtk::Accessible, gtk::Buildable, gtk::ConstraintTarget;
 }
 
 impl PpsSidebarLinks {
