@@ -212,8 +212,7 @@ mod imp {
             let document = self.document_view.model().document().unwrap();
 
             imp.document_view.set_filenames(display_name, edit_name);
-            imp.document_view
-                .open_document(&document, metadata, dest, WindowRunMode::Normal);
+            imp.document_view.open_document(&document, metadata, dest);
             imp.set_mode(WindowRunMode::Normal);
 
             win.set_default_size(self.obj().width(), self.obj().height());
@@ -246,7 +245,7 @@ mod imp {
             };
             let secondary_text_command =
                 gettext("If you reload the document, changes will be permanently lost.");
-            let text = gettext("File changed outside Papers. Reload document?");
+            let text = gettext("File changed outside Document Viewer. Reload document?");
 
             let dialog = adw::AlertDialog::builder()
                 .body(format!("{secondary_text} {secondary_text_command}"))
@@ -798,19 +797,6 @@ mod imp {
                         obj.default_settings
                             .bind("window-maximized", &window, "maximized")
                             .build();
-                        // When the window is resized, the sidebar may incorrectly grab the focus
-                        glib::timeout_add_local_once(
-                            std::time::Duration::from_millis(100),
-                            glib::clone!(
-                                #[weak(rename_to = obj)]
-                                obj,
-                                move || {
-                                    if obj.mode.get() != WindowRunMode::Presentation {
-                                        obj.document_view.focus_view()
-                                    }
-                                }
-                            ),
-                        );
                     }
                 ),
             );
@@ -894,7 +880,6 @@ mod imp {
                                 &document,
                                 obj.metadata.borrow().clone().as_ref(),
                                 dest.as_ref(),
-                                pending_mode,
                             );
 
                             obj.document_view
@@ -935,12 +920,7 @@ mod imp {
                                 #[weak]
                                 job,
                                 async move {
-                                    if e.matches(papers_document::DocumentError::Encrypted)
-                                        && job
-                                            .loaded_document()
-                                            .and_dynamic_cast::<papers_document::DocumentSecurity>()
-                                            .is_ok()
-                                    {
+                                    if e.matches(papers_document::DocumentError::Encrypted) {
                                         obj.set_mode(WindowRunMode::PasswordView);
 
                                         // First look whether password is in keyring
@@ -999,69 +979,22 @@ mod imp {
                 #[weak(rename_to = obj)]
                 self,
                 async move {
-                    // Check if the file comes from a remote GVFS backend (like Google Drive, SMB, SFTP, etc.)
-                    // These usually need to be copied locally to avoid access or seek issues.
-                    let is_gvfs = file
-                        .uri_scheme()
-                        .map(|scheme| {
-                            // List of GVFS backends that should be copied locally:
-                            // - Network: smb, ftp, sftp, dav/davs, nfs
-                            // - Cloud: google-drive, onedrive
-                            // - Mobile: afc (ios), mtp (android)
-                            // - Other: afp (Apple Filing Protocol)
-                            matches!(
-                                scheme.as_str(),
-                                "afc"
-                                    | "afp"
-                                    | "dav"
-                                    | "davs"
-                                    | "ftp"
-                                    | "google-drive"
-                                    | "mtp"
-                                    | "nfs"
-                                    | "onedrive"
-                                    | "sftp"
-                                    | "smb"
-                            )
-                        })
-                        .unwrap_or(false);
-
-                    // If the file doesn't have a normal local path or if it's remote file,
-                    // then copy it locally before opening to avoid permission or access problems.
-                    if path.is_none() || is_gvfs {
+                    if path.is_none() {
                         obj.load_remote_file(file.as_ref()).await;
                     } else {
-                        // File appears to be local, but verify it's actually seekable.
-                        // Some FUSE-backed GVFS filesystems may provide paths but still have seekability issues
+                        // source_file is probably local, but make sure it's seekable
+                        // before loading it directly.
                         let result = file.read_future(glib::Priority::DEFAULT).await;
 
-                        match result {
-                            Ok(stream) => {
-                                // Check If the stream doesn’t support seeking at all, then treat it as remote.
-                                if !stream.can_seek() {
-                                    return obj.load_remote_file(file.as_ref()).await;
-                                }
-
-                                // Verify that G_SEEK_END works because
-                                // some GVFS backends claim to support seeking, but fail when using G_SEEK_END.
-                                let seekable = stream.upcast_ref::<gio::Seekable>();
-                                if seekable
-                                    .seek(0, glib::SeekType::End, gio::Cancellable::NONE)
-                                    .is_err()
-                                {
-                                    return obj.load_remote_file(file.as_ref()).await;
-                                }
-
-                                // If stream is properly seekable, load directly
-                                obj.loader_view.set_fraction(-1_f64);
-                                obj.set_mode(WindowRunMode::LoaderView);
-                                load_job.scheduler_push_job(papers_view::JobPriority::PriorityNone);
-                            }
-                            Err(_) => {
-                                // If it failed to open stream, try copying to local
-                                obj.load_remote_file(file.as_ref()).await;
+                        if let Ok(stream) = result {
+                            if !stream.can_seek() {
+                                return obj.load_remote_file(file.as_ref()).await;
                             }
                         }
+
+                        obj.loader_view.set_fraction(-1_f64);
+                        obj.set_mode(WindowRunMode::LoaderView);
+                        load_job.scheduler_push_job(papers_view::JobPriority::PriorityNone);
                     }
                 }
             ));
@@ -1070,7 +1003,7 @@ mod imp {
         fn set_night_mode(&self, night_mode: bool) {
             self.document_view.set_inverted_colors(night_mode);
 
-            let manager = adw::StyleManager::for_display(&self.obj().display());
+            let manager = adw::StyleManager::for_display(&WidgetExt::display(&self.obj().clone()));
 
             let color_scheme = if night_mode {
                 adw::ColorScheme::ForceDark
@@ -1212,7 +1145,7 @@ mod imp {
         }
 
         fn launch_external_uri(&self, action: &LinkAction) {
-            let context = self.obj().display().app_launch_context();
+            let context = WidgetExt::display(&self.obj().clone()).app_launch_context();
             let uri = action.uri().unwrap();
             let file = gio::File::for_uri(&uri);
 
@@ -1283,7 +1216,7 @@ mod imp {
 glib::wrapper! {
     pub struct PpsWindow(ObjectSubclass<imp::PpsWindow>)
         @extends gtk::Widget, gtk::Window, gtk::ApplicationWindow, adw::ApplicationWindow,
-        @implements gtk::Native, gio::ActionGroup, gio::ActionMap;
+        @implements gtk::Native, gio::ActionGroup, gio::ActionMap, gtk::Accessible, gtk::Buildable, gtk::ConstraintTarget, gtk::Root, gtk::ShortcutManager;
 }
 
 impl PpsWindow {
